@@ -131,6 +131,11 @@ void unlink_node(linestruct *fileptr)
 /* Free the data structures in the given node. */
 void delete_node(linestruct *fileptr)
 {
+#ifdef ENABLE_WRAPPING
+	/* If the spill-over line for hard-wrapping is deleted... */
+	if (fileptr == openfile->spillage_line)
+		openfile->spillage_line = NULL;
+#endif
 	free(fileptr->data);
 #ifdef ENABLE_COLOR
 	free(fileptr->multidata);
@@ -493,13 +498,12 @@ void copy_from_buffer(linestruct *somebuffer)
 /* Unlink a node from the rest of the circular list, and delete it. */
 void unlink_opennode(openfilestruct *fileptr)
 {
-#ifdef ENABLE_MULTIBUFFER
-	if (fileptr == firstfile)
-		firstfile = firstfile->next;
+	if (fileptr == startfile)
+		startfile = startfile->next;
 
 	fileptr->prev->next = fileptr->next;
 	fileptr->next->prev = fileptr->prev;
-#endif
+
 	delete_opennode(fileptr);
 }
 
@@ -516,7 +520,7 @@ void delete_opennode(openfilestruct *fileptr)
 #endif
 	free(fileptr);
 }
-#endif
+#endif /* ENABLE_MULTIBUFFER */
 
 /* Display a warning about a key disabled in view mode. */
 void print_view_warning(void)
@@ -737,13 +741,13 @@ void mouse_init(void)
 void print_opt(const char *shortflag, const char *longflag, const char *desc)
 {
 	printf(" %s\t", shortflag);
-	if (strlenpt(shortflag) < 8)
+	if (breadth(shortflag) < 8)
 		printf("\t");
 
 	printf("%s\t", longflag);
-	if (strlenpt(longflag) < 8)
+	if (breadth(longflag) < 8)
 		printf("\t\t");
-	else if (strlenpt(longflag) < 16)
+	else if (breadth(longflag) < 16)
 		printf("\t");
 
 	printf("%s\n", _(desc));
@@ -1781,9 +1785,6 @@ void do_input(void)
 		} else
 #endif
 		{
-#ifdef ENABLE_WRAPPING
-			linestruct *was_next = openfile->current->next;
-#endif
 #ifndef NANO_TINY
 			linestruct *was_current = openfile->current;
 			size_t was_x = openfile->current_x;
@@ -1813,14 +1814,6 @@ void do_input(void)
 					also_the_last = FALSE;
 			}
 #endif
-#ifdef ENABLE_WRAPPING
-			/* If the cursor moved to another line and this was not caused
-			 * by adding characters to the buffer, clear the prepend flag. */
-			if (openfile->current->next != was_next &&
-							shortcut->func != do_tab &&
-							shortcut->func != do_verbatim_input)
-				wrap_reset();
-#endif
 #ifdef ENABLE_COLOR
 			if (!refresh_needed && !okay_for_view(shortcut))
 				check_the_multis(openfile->current);
@@ -1920,7 +1913,7 @@ void do_output(char *output, size_t output_len, bool allow_cntrls)
 
 #ifdef ENABLE_WRAPPING
 		/* If text gets wrapped, the edit window needs a refresh. */
-		if (ISSET(BREAK_LONG_LINES) && do_wrap(openfile->current))
+		if (ISSET(BREAK_LONG_LINES) && do_wrap())
 			refresh_needed = TRUE;
 #endif
 	}
@@ -2437,14 +2430,14 @@ int main(int argc, char **argv)
 		/* Simply OR the boolean flags from rcfile and command line. */
 		for (size_t i = 0; i < sizeof(flags) / sizeof(flags[0]); i++)
 			flags[i] |= flags_cmdline[i];
-
-		if (hardwrap == 0)
-			UNSET(BREAK_LONG_LINES);
-		else if (hardwrap == 1)
-			SET(BREAK_LONG_LINES);
 	}
 #endif /* ENABLE_NANORC */
 
+	if (hardwrap == 0)
+		UNSET(BREAK_LONG_LINES);
+	else if (hardwrap == 1)
+		SET(BREAK_LONG_LINES);
+
 	/* If the user wants bold instead of reverse video for hilited text... */
 	if (ISSET(BOLD_TEXT))
 		hilite_attribute = A_BOLD;
